#include "STM32F4xx.h"

#define LCD_COLUMNS   8   // Number of LCD columns in characters
#define LCD_ROWS     	2   // Number of LCD rows
#define delay_20_ms   700000
#define delay_2_ms    70000

/*-------------------- LCD interface hardware definitions --------------------*/

/* Connections from LCD to MCU port bits: 
 DB4 through DB8 are contiguous, starting with LSB at bit position PIN_DATA_SHIFT
 
  For example:
   - DB4 = PD0
   - DB5 = PD1
   - DB6 = PD2
   - DB7 = PD3
	 
   - E   = PD4
   - RW  = PD7
   - RS  = PD6                                                              */

#define PIN_DATA_PORT					GPIOD
#define PIN_DATA_PT						GPIOD
#define PIN_DATA_SHIFT				( 0 )

#define PINS_DATA             (0xf<< (PIN_DATA_SHIFT))


#define PIN_DATA_PORT_4				GPIOD
#define PIN_DATA_PT_4					GPIOD
#define PIN_DATA_SHIFT_4			( 0 )
#define PIN_DATA_4            ( 1 << PIN_DATA_SHIFT_4)
#define PIN_DATA_4_01         (01 << (PIN_DATA_SHIFT_4*2))
#define PIN_DATA_4_11         (11 << (PIN_DATA_SHIFT_4*2))

#define PIN_DATA_PORT_5				GPIOD
#define PIN_DATA_PT_5					GPIOD
#define PIN_DATA_SHIFT_5			( 1 )
#define PIN_DATA_5            ( 1 << PIN_DATA_SHIFT_5)
#define PIN_DATA_5_01         (01 << (PIN_DATA_SHIFT_5*2))
#define PIN_DATA_5_11         (11 << (PIN_DATA_SHIFT_5*2))


#define PIN_DATA_PORT_6				GPIOD
#define PIN_DATA_PT_6					GPIOD
#define PIN_DATA_SHIFT_6			( 2 )
#define PIN_DATA_6            ( 1 << PIN_DATA_SHIFT_6)
#define PIN_DATA_6_01         (01 << (PIN_DATA_SHIFT_6*2))
#define PIN_DATA_6_11         (11 << (PIN_DATA_SHIFT_6*2))

#define PIN_DATA_PORT_7				GPIOD
#define PIN_DATA_PT_7					GPIOD
#define PIN_DATA_SHIFT_7			( 3 )
#define PIN_DATA_7            ( 1 << PIN_DATA_SHIFT_7)
#define PIN_DATA_7_01         (01 << (PIN_DATA_SHIFT_7*2))
#define PIN_DATA_7_11         (11 << (PIN_DATA_SHIFT_7*2))


#define PIN_E_PORT						GPIOD
#define PIN_E_PT							GPIOD
#define PIN_E_SHIFT						( 4 )
#define PIN_E                 ( 1 << PIN_E_SHIFT)
#define PIN_E_01              (01 << (PIN_E_SHIFT*2))

#define PIN_RW_PORT						GPIOD
#define PIN_RW_PT							GPIOD
#define PIN_RW_SHIFT					( 7 )
#define PIN_RW                ( 1 << PIN_RW_SHIFT)
#define PIN_RW_01             (01 << (PIN_RW_SHIFT*2))

#define PIN_RS_PORT						GPIOD
#define PIN_RS_PT							GPIOD
#define PIN_RS_SHIFT	        ( 6 )
#define PIN_RS                ( 1 << PIN_RS_SHIFT)
#define PIN_RS_01             (01 << (PIN_RS_SHIFT*2))



/* Enable Clock for peripheral driving LCD pins                               */
#define ENABLE_LCD_PORT_CLOCKS   	RCC->AHB1ENR|=RCC_AHB1ENR_GPIODEN;	

#define SET_LCD_E(x)              if (x) {PIN_E_PT->BSRRL = PIN_E;} else {PIN_E_PT->BSRRH = PIN_E;}
#define SET_LCD_RW(x)             if (x) {PIN_RW_PT->BSRRL = PIN_RW;} else {PIN_RW_PT->BSRRH= PIN_RW;}
#define SET_LCD_RS(x)             if (x) {PIN_RS_PT->BSRRL = PIN_RS;} else {PIN_RS_PT->BSRRH = PIN_RS;}

#define SET_LCD_DATA_OUT(x)       PIN_DATA_PT->ODR = (PIN_DATA_PT->ODR & ~(PINS_DATA)) | (x<<PIN_DATA_SHIFT);
//PIN_DATA_PT->ODR = (PIN_DATA_PT->ODR & ~PINS_DATA) | ((x) << (PIN_DATA_SHIFT));
#define GET_LCD_DATA_IN           (((PIN_DATA_PT->IDR & PINS_DATA) >> PIN_DATA_SHIFT) & 0x0F)

/* Setting all pins to output mode                                            */
#define SET_LCD_ALL_DIR_OUT     { PIN_DATA_PT->MODER = PIN_DATA_PT->MODER | PIN_DATA_4_01; \
																PIN_DATA_PT->MODER = PIN_DATA_PT->MODER | PIN_DATA_5_01; \
																PIN_DATA_PT->MODER = PIN_DATA_PT->MODER | PIN_DATA_6_01; \
																PIN_DATA_PT->MODER = PIN_DATA_PT->MODER | PIN_DATA_7_01; \
																PIN_E_PT->MODER = PIN_E_PT->MODER | (PIN_E_01); \
																PIN_RW_PT->MODER = PIN_RW_PT->MODER | (PIN_RW_01); \
																PIN_RS_PT->MODER = PIN_RS_PT->MODER | (PIN_RS_01); \
																											}

/* Setting DATA pins to input mode                                            */
#define SET_LCD_DATA_DIR_IN       PIN_DATA_PT->MODER = PIN_DATA_PT->MODER & ~\
																	(PIN_DATA_4_11|PIN_DATA_5_11|PIN_DATA_6_11|PIN_DATA_7_11);

/* Setting DATA pins to output mode                                           */
#define SET_LCD_DATA_DIR_OUT    { PIN_DATA_PT->MODER = PIN_DATA_PT->MODER | PIN_DATA_4_01; \
																PIN_DATA_PT->MODER = PIN_DATA_PT->MODER | PIN_DATA_5_01; \
																PIN_DATA_PT->MODER = PIN_DATA_PT->MODER | PIN_DATA_6_01; \
																PIN_DATA_PT->MODER = PIN_DATA_PT->MODER | PIN_DATA_7_01; \
																	}



#define LCD_BUSY_FLAG_MASK				(0x80)

/******************************************************************************/
void Delay (uint32_t nCount);
void Init_LCD (void);
void Set_Cursor (uint8_t column, uint8_t row);
void Clear_LCD(void);
void Print_LCD (char *string);
void lcd_putchar (char c);

// *******************************ARM University Program Copyright  ARM Ltd 2013*************************************   
