


#include "STM32F4xx.h"
#include "LCD_4bit.h"

#define MASK(x) (1UL << (x))
#define LOOP_COUNT 0x3fffff


/*------------------------------------------------------------------------------
	Example for initialising 4 on board LEDs
	*------------------------------------------------------------------------------*/
void Init_LEDs(void){
		
		//Enable clock to port D		
		RCC->AHB1ENR|=RCC_AHB1ENR_GPIODEN;

	
		//Configure the pins which are conenected to the on board LEDs
		//to be output
		GPIOD->MODER|=(GPIO_MODER_MODER12_0|//Green
									 GPIO_MODER_MODER13_0|//Orange
									 GPIO_MODER_MODER14_0|//Red
									 GPIO_MODER_MODER15_0);//Blue

}


/*------------------------------------------------------------------------------
	Example for initialising an external speaker
	Connect the speaker (with a cpacitor and optional resistor) to port D pin 2
	and the GND
	*------------------------------------------------------------------------------*/
void Init_Speaker(void){

		//Enable clock to port D
		RCC->AHB1ENR|=RCC_AHB1ENR_GPIODEN;
	
		GPIOD->MODER|=GPIO_MODER_MODER2_0;
	


}

/*------------------------------------------------------------------------------
	Making various sounds depending on the frequency
	*------------------------------------------------------------------------------*/

void Speaker_Beep(uint32_t frequency){
	
	Init_Speaker();

	while(1){
			GPIOD->BSRRL=(MASK(2));
			Delay(frequency);
			GPIOD->BSRRH=(MASK(2));
			Delay(frequency);
	}

}

/*------------------------------------------------------------------------------
	Turn the led on and off and repeat
	*------------------------------------------------------------------------------*/

void Blinking_Lights (void) {
	
		Init_LEDs();
		
		while(1){
			//Turn light on
			GPIOD->BSRRL=(MASK(12));
			Delay(LOOP_COUNT);
			//Turn light off
			GPIOD->BSRRH=(MASK(12));
			Delay(LOOP_COUNT);
			GPIOD->BSRRL=(MASK(13));
			Delay(LOOP_COUNT);
			GPIOD->BSRRH=(MASK(13));
			Delay(LOOP_COUNT);
			GPIOD->BSRRL=(MASK(14));
			Delay(LOOP_COUNT);
			GPIOD->BSRRH=(MASK(14));
			Delay(LOOP_COUNT);
			GPIOD->BSRRL=(MASK(15));
			Delay(LOOP_COUNT);
			GPIOD->BSRRH=(MASK(15));
			Delay(LOOP_COUNT);

		}

}


/*------------------------------------------------------------------------------
	Example for switching LEDs using external switch
	Connect the switches to Port A pin 7 and the GND
	*------------------------------------------------------------------------------*/
void 	Basic_Light_Switching_Example(void){
			
		Init_LEDs();

		//Enable clock to port A
		RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN;
	
		//Pull-up the output
		GPIOA->PUPDR|=GPIO_PUPDR_PUPDR7_0; 
	
		while(1){
			//See whether the switch is on (or the button is pressed)
			//If pressed, then the pin should be grounded, then turn the lights on
			if(!(GPIOA->IDR&(MASK(7)))){
				GPIOD->BSRRL=(MASK(12));
				GPIOD->BSRRL=(MASK(13));
				GPIOD->BSRRL=(MASK(14));
				GPIOD->BSRRL=(MASK(15));
			}
			//If not, then the pin should be high, then turn the lights off
			else{
				GPIOD->BSRRH=(MASK(12));
				GPIOD->BSRRH=(MASK(13));
				GPIOD->BSRRH=(MASK(14));
				GPIOD->BSRRH=(MASK(15));
			}
		}			

}


/*------------------------------------------------------------------------------
	Example for Using LCD module
	*------------------------------------------------------------------------------*/

void LCD_Example(void) {
	Init_LCD();
	Clear_LCD();
	Set_Cursor(0,0);
	Print_LCD("Hi,World!");
	Set_Cursor(0,1);
	Print_LCD("LCD Test");
}


int main(void)
{
	
	//Uncomment to run a demo

	Blinking_Lights();
	
//	Basic_Light_Switching_Example();
	
//	Speaker_Beep(20000);
	
//	LCD_Example();


}

// *******************************ARM University Program Copyright  ARM Ltd 2013*************************************   
